%
%   MODELDEMO.M   Sequence of Matlab sample programs that demonstrate  
%              various modeling and simulation techniques (both time and 
%              frequency domain) including a simple Simulink sample 
%
%   Mass-Spring-Dashpot Mechanical System
%     my''(t) + cy'(t) + ky(t)  =  u(t)
%
%   This example has been broken into three parts (3 files), as follows:
%    Demo 1: simple simulation and model conversion techniques   (*** this file ***)
%    Demo 2: modeling of systems in block diagram form (the OLD way)
%    Demo 3: modeling with Simulink (the NEW way)
%
%   File prepared by J. R. White, UMass-Lowell (last update: Feb. 2020)
%

      clear all, close all, nfig = 0;
      format compact
%
%   open diary file for saving solutions
      delete modeldemo1.out,    diary modeldemo1.out
      disp('  *** MODELDEMO1.OUT  ***   Diary File for MODELDEMO1.M '),   disp(' ')
%
%   define coefficients for several cases
      m = 1                   % kg      mass
      k = 1                   % kg/s^2  spring const
      c = [2.0 2.5 1.2 0.0]   % kg/s    critical/over/under/no damping factor
%
%   create state space model for each case
      disp('  State Space Models')
      km = k/m;
      A1 = [0 1;-km -c(1)/m],     A2 = [0 1;-km -c(2)/m]
      A3 = [0 1;-km -c(3)/m],     A4 = [0 1;-km -c(4)/m]
      B = [0 1/m]',     C = [1 0],     D = [0]
      sys1s = ss(A1,B,C,D);     sys2s = ss(A2,B,C,D);
      sys3s = ss(A3,B,C,D);     sys4s = ss(A4,B,C,D);
%
%   simulate different time domain responses (impulse & step inputs)
      t = 0:.2:15;            % time vector (seconds)
%
      y1 = impulse(sys1s,t);  y2 = impulse(sys2s,t); 
      y3 = impulse(sys3s,t);  y4 = impulse(sys4s,t);
      nfig = nfig+1;  figure(nfig)
      subplot(221),plot(t,y1,'r','LineWidth',2),title('Critical Damping - Impulse'),grid
      xlabel('Time (sec)'),ylabel('System Response')
      subplot(222),plot(t,y2,'b','LineWidth',2),title('Over Damping - Impulse'),grid
      xlabel('Time (sec)'),ylabel('System Response')
      subplot(223),plot(t,y3,'g','LineWidth',2),title('Under Damping - Impulse'),grid
      xlabel('Time (sec)'),ylabel('System Response')
      subplot(224),plot(t,y4,'m','LineWidth',2),title('No Damping - Impulse'),grid
      xlabel('Time (sec)'),ylabel('System Response')
%
      y1 = step(sys1s,t);  y2 = step(sys2s,t);
      y3 = step(sys3s,t);  y4 = step(sys4s,t);
      nfig = nfig+1;  figure(nfig)
      subplot(221),plot(t,y1,'r','LineWidth',2),title('Critical Damping - Step'),grid
      xlabel('Time (sec)'),ylabel('System Response')
      subplot(222),plot(t,y2,'b','LineWidth',2),title('Over Damping - Step'),grid
      xlabel('Time (sec)'),ylabel('System Response')
      subplot(223),plot(t,y3,'g','LineWidth',2),title('Under Damping - Step'),grid
      xlabel('Time (sec)'),ylabel('System Response')
      subplot(224),plot(t,y4,'m','LineWidth',2),title('No Damping - Step'),grid
      xlabel('Time (sec)'),ylabel('System Response')
%
%   Model Conversion in Matlab
%
%   For the case with m = k = 1 (numerically), the system transfer function
%   for the simple mass-spring-dashpot system is given as
%         G(s)  =  1/[s^2 + cs + 1)]   =  z(s)/p(s)
%   for various values of c (viscous damping factor)
%
%   State-Space to Transfer Function in Matlab
      disp('  Transfer Function Form')
      sys1t = tf(sys1s),   sys2t = tf(sys2s)
      sys3t = tf(sys3s),   sys4t = tf(sys4s)
%
%   Transfer Function to Zero-Pole in MATLAB
      disp('  Zero-Pole-Gain Form')
      sys1z = zpk(sys1t),   sys2z = zpk(sys2t)
      sys3z = zpk(sys3t),   sys4z = zpk(sys4t)
%
%   Sometimes the Residue or Partial Fraction form is useful
%   (not really in this case, but let's demonstrate it anyway)
      disp('  Residue Form')
%   note that the tfdata command defines cell arrays
      [n1,d1] = tfdata(sys1t);   [n2,d2] = tfdata(sys2t); 
      [n3,d3] = tfdata(sys3t);   [n4,d4] = tfdata(sys4t);
%   let's extract the cell arrays into normal vectors for use in residue
      n1 = n1{1}, d1 = d1{1},     n2 = n2{1}, d2 = d2{1}, 
      n3 = n3{1}, d3 = d3{1},     n4 = n4{1}, d4 = d4{1}, 
%   now let's use the residue command
      [r1,pr1,kr1] = residue(n1,d1),   [r2,pr2,kr2] = residue(n2,d2)
      [r3,pr3,kr3] = residue(n3,d3),   [r4,pr4,kr4] = residue(n4,d4)
%
%   Getting Frequency Domain Plots is also easy in Matlab
%
%   Bode plots
      nfig = nfig+1;  figure(nfig)
      w = logspace(-2,2,400);
      subplot(2,2,1),bode(sys1s,'r',w),title('Critical Damping'),grid     % ss form
      subplot(2,2,2),bode(sys2s,'b',w),title('Over Damping'),grid         % ss form
      subplot(2,2,3),bode(sys3t,'g',w),title('Under Damping'),grid        % tf form
      subplot(2,2,4),bode(sys4t,'m',w),title('No Damping'),grid           % tf form
      h = findobj(gcf,'LineStyle','-'); nh = length(h); 
      for i = 1:nh; set(h(i),'LineWidth',2); end
%
%   Nichols plots (log-mag vs angle)
      nfig = nfig+1;  figure(nfig)
      subplot(2,2,1),nichols(sys1s,'r',w),grid,title('Critical Damping')
      subplot(2,2,2),nichols(sys2s,'b',w),grid,title('Over Damping')
      subplot(2,2,3),nichols(sys3s,'g',w),grid,title('Under Damping')
      subplot(2,2,4),nichols(sys4s,'m',w),grid,title('No Damping')
      h = findobj(gcf,'LineStyle','-'); nh = length(h); 
      for i = 1:nh; set(h(i),'LineWidth',2); end
%
%   Nyquist plots (Im vs Re)
      nfig = nfig+1;  figure(nfig)
      subplot(2,2,1),nyquist(sys1s,'r',w),grid,title('Critical Damping')
      subplot(2,2,2),nyquist(sys2s,'b',w),grid,title('Over Damping')
      subplot(2,2,3),nyquist(sys3s,'g',w),grid,title('Under Damping')
      subplot(2,2,4),nyquist(sys4s,'m',w),grid,title('No Damping')
      h = findobj(gcf,'LineStyle','-'); nh = length(h); 
      for i = 1:nh; set(h(i),'LineWidth',2); end
%
      diary off                % turn off diary file
%
%   Note: Before leaving this demo, briefly illustrate the LTIVIEW command!!!
%   (use File/Import to bring in the systems of interest -- then explore...)
%   (note also that all the above frequency domain plots are "interactive")
      ltiview
%
%   end of demo
